require ("math")

function love.load()
	screenWidth = 800
    screenHeight = 600
	love.window.setMode(screenWidth,screenHeight)
	love.graphics.setBackgroundColor(40,40,80)

	meter = (64) -- our scale for the physics world

	world = love.physics.newWorld(0, 9.81*meter, true)
	world:setCallbacks(beginContact, endContact, preSolve, postSolve)

    rng = love.math.newRandomGenerator()
    rng:setSeed(os.time())

	objects = {}
	--re-useable colors
	red    = {200,20 ,20 }
	green  = {20 ,200,20 }
	blue   = {20 ,20 ,200}
	yellow = {200,200,20 }
	orange = {230,110,20 }
	pink   = {200,20 ,200}
	cyan   = {20 ,200,200}

	grey   = {150,150,150}
	glowing= {0  ,0  ,0  }
	glowIntensity = 0
	runTime = 0
	nextColor = randomColor()

	blockShape = love.physics.newRectangleShape(0, 0, 50, 50)
	circleShape = love.physics.newCircleShape(25)

	timescale = 0.9/8

	score = 0
	lostObjects = 0
	lostLimit = 4
	lostObjects = 1
	gameover = false

	totalGain = 0
	totalLoss = 0

	--let's create the ground :D :D :D :D
	ground = {}
	ground.body = love.physics.newBody(world, screenWidth/2, screenHeight) --physics bodies orient by their center
	ground.shape = love.physics.newRectangleShape(500, 120) --make a rectangle with a width of 650 and a height of 50
	ground.fixture = love.physics.newFixture(ground.body, ground.shape); --attach shape to body
	ground.fixture:setUserData("ground") --differentiate ground from others

	sound1 = love.audio.newSource("good2.wav", "static") --score up
	sound2 = love.audio.newSource("bad.wav",   "static") --lost block
	sound3 = love.audio.newSource("blip.wav",  "static") --place block
	sound4 = love.audio.newSource("ded.wav",   "static") --game over
	sound5 = love.audio.newSource("zap.wav",   "static") --glowy noise

	--this code would start the game with block on-screen already:
	--for x = 266,553,54 do --6 blocks wide
	--	for y = 300,350,50 do --2 blocks high
	--		addBlock(x,y,randomColor())
	--	end
	--end
end

function addBlock(x,y,color)
	local color = color
	newid = table.getn(objects)+1
	objects[newid] = {}
	objects[newid].body = love.physics.newBody(world, x, y, "dynamic")
	objects[newid].shape = blockShape
	objects[newid].fixture = love.physics.newFixture(objects[newid].body, objects[newid].shape,5)
	objects[newid].color = color
	objects[newid].matching = 0
	objects[newid].dying = -1
	objects[newid].fixture:setRestitution(0.3)
	objects[newid].fixture:setFriction(0.135)
	objects[newid].fixture:setUserData(newid) -- used in collision callbacks so each object can have a corresponding position in this array
end

function love.update(dt)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	for i, obj in pairs(objects) do
		if obj ~= nil then
			if obj.dying > -1 then
				objects[i].dying = objects[i].dying + dt -- Objects don't clear until after a timer is up
				if obj.dying > 1.5 then
					objects[i].body:destroy()
					objects[i] = nil
					if gameover == false then
						score = score + 100
						totalGain = totalGain + 100
						print("gain:"..totalGain)
						lostObjects = lostObjects - 0.05
						sound1:stop()
						sound1:play()
					end
					if lostObjects < 0 then lostObjects = 0 end
				end
			else
				x, y = objects[i].body:getPosition( ) -- Objects that fall off without being matched are counted
				if y > 600 then
					objects[i].body:destroy()
					objects[i] = nil
					if gameover == false then
						lostObjects = lostObjects + 1
						score = score - 100
						totalLoss = totalLoss + 100
						sound2:stop()
						sound2:play()
						print("loss:"..totalLoss)
					end
				end
			end
		end
	end
	if lostObjects >= lostLimit and gameover == false then
		gameover = true
		sound4:play()
	end --game ends when too many blocks fall off the screen

	runTime = runTime + dt
	glowIntensity = 205 + (50*math.sin(runTime*6))	--make stuff pulsate :D
	--print(glowIntensity)
	glowing = {glowIntensity, glowIntensity, glowIntensity}
end

function love.draw()
	love.graphics.setLineWidth(3)
	love.graphics.setColor(72, 160, 14) -- set the drawing color to green for the ground
	love.graphics.polygon("fill", ground.body:getWorldPoints(ground.shape:getPoints())) -- draw a "filled in" polygon using the ground's coordinates
	love.graphics.setColor(red)
	love.graphics.rectangle("fill",0,580,screenWidth-(screenWidth*(lostObjects/lostLimit)),20) --Basically a health bar
	for i, obj in pairs(objects) do
		if obj ~= nil then
			love.graphics.setColor(obj.color)
			love.graphics.polygon("fill", obj.body:getWorldPoints(obj.shape:getPoints()))
			if obj.dying > -1 then love.graphics.setColor(glowing) else love.graphics.setColor(0,0,0) end --color text based on whether the block has been matched
			love.graphics.polygon("line", obj.body:getWorldPoints(obj.shape:getPoints()))
			--love.graphics.print(obj.matching, obj.body:getWorldCenter())-- would print the number of touching shapes, on the shape
		end
	end
	love.graphics.setColor(grey)
	love.graphics.rectangle("line",love.mouse.getX()-25, love.mouse.getY()-25, 50, 50)
	if gameover then
		love.graphics.setColor(glowing)
		love.graphics.print("Game over, man!", 130, 200, 0, 5)
		love.graphics.setColor(255,255,255)
		love.graphics.print("SCORE:"..score, 200, 270, 0, 4)--draw score in different location on gameover
		print("gain:"..totalGain)
		print("loss:"..totalLoss)
	else
		love.graphics.setColor(255,255,255)
		love.graphics.print("SCORE:"..score, 400, 20, 0, 4)
	end
end

function love.mousepressed(x,y,button)
	if gameover == false then
		addBlock(x,y,nextColor)
		nextColor = randomColor()
		sound3:stop()
		sound3:play()
	end
end

function beginContact(a,b,coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			objects[dataA].matching = objects[dataA].matching + 1
			objects[dataB].matching = objects[dataB].matching + 1
		end
	end
end

function endContact(a,b,coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			objects[dataA].matching = objects[dataA].matching - 1
			objects[dataB].matching = objects[dataB].matching - 1
		end
	end
end

function postSolve(a, b, coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	local sound = false
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			if objects[dataA].matching > 1 or objects[dataB].matching > 1 or objects[dataA].dying > -1 or objects[dataB].dying > -1 then --Clear groups of 3 or more
				if objects[dataA].dying == -1 then
					objects[dataA].dying = 0
					sound = true
				end
				if objects[dataB].dying == -1 then
					objects[dataB].dying = 0
					sound = true
				end
			end
		end
	end
	if sound then 
		sound5:stop()
		sound5:play() end
end

function love.keypressed(key)

end

function randomColor()
	local c = {}
	local n = math.floor(rng:random(0,5.9999))
	if n == 0 then c = red end
	if n == 1 then c = green end
	if n == 2 then c = blue end
	if n == 3 then c = cyan end
	if n == 4 then c = pink end
	if n == 5 then c = yellow end
--	if n == 6 then c = orange end
	return c
end